# frozen_string_literal: true

# Released under the MIT License.
# Copyright, 2016-2025, by Samuel Williams.
# Copyright, 2018, by Gabriel Mazetto.

# @namespace
module Samovar
	VERSION = "2.5.1"
end
